@@ -4,7 +4,7 @@
  *
  *   Copyright (C) 2000-2003 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: gerbv.c,v 1.127 2003/03/02 16:14:25 spetm Exp $
+ * $Id: gerbv.c,v 1.140 2004/05/19 20:01:17 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
@@ -47,6 +48,9 @@
 #include <getopt.h>
 #endif
 
+#ifdef USE_GTK2
+#include <pango/pango.h>
+#endif
 
 #include "gerber.h"
 #include "drill.h"
@@ -57,9 +61,11 @@
 #include "gerbv_icon.h"
 #include "log.h"
 #include "setup.h"
+#include "project.h"
 #ifdef EXPORT_PNG
 #include "exportimage.h"
 #endif /* EXPORT_PNG */
+#include "tooltable.h"
 
 #define WIN_TITLE "Gerber Viewer : "
 
@@ -78,16 +84,23 @@ typedef struct {
  */
 gerbv_screen_t screen;
 
+#define SAVE_PROJECT 0
+#define SAVE_AS_PROJECT 1
+#define OPEN_PROJECT 2
 
 static gint expose_event (GtkWidget *widget, GdkEventExpose *event);
 static void draw_zoom_outline(gboolean centered);
 static void draw_measure_distance();
 static void swap_layers(GtkWidget *widget, gpointer data);
 static void color_selection_popup(GtkWidget *widget, gpointer data);
+static void invert_color(GtkWidget *widget, gpointer data);
 static void load_file_popup(GtkWidget *widget, gpointer data);
 #ifdef EXPORT_PNG
 static void export_png_popup(GtkWidget *widget, gpointer data);
 #endif /* EXPORT_PNG */
+static void cb_ok_project(GtkWidget *widget, gpointer data);
+static void project_popup(GtkWidget *widget, gpointer data);
+static void project_save_cb(GtkWidget *widget, gpointer data);
 static void unload_file(GtkWidget *widget, gpointer data);
 static void reload_files(GtkWidget *widget, gpointer data);
 static void menu_zoom(GtkWidget *widget, gpointer data);
@@ -95,6 +108,7 @@ static void si_func(GtkWidget *widget, gpointer data);
 static void unit_func(GtkWidget *widget, gpointer data);
 static void all_layers_on(GtkWidget *widget, gpointer data);
 static void all_layers_off(GtkWidget *widget, gpointer data);
+static void load_project(project_list_t *project_list);
 static void zoom(GtkWidget *widget, gpointer data);
 static void zoom_outline(GtkWidget *widget, GdkEventButton *event);
 static gint redraw_pixmap(GtkWidget *widget, int restart);
@@ -108,6 +122,7 @@ GtkWidget *lookup_widget (GtkWidget * widget, const gchar * widget_name);
 static void zoom_ok_button_clicked (GtkButton * button, gpointer user_data);
 static void zoom_cancel_button_clicked (GtkButton * button, gpointer user_data);
 
+void rename_main_window(char *filename, GtkWidget *main_win);
 
 void
 destroy(GtkWidget *widget, gpointer data)
@@ -141,6 +156,10 @@ destroy(GtkWidget *widget, gpointer data)
 
 static GtkItemFactoryEntry menu_items[] = {
     {"/_File",               NULL,     NULL,             0, "<Branch>"},
+    {"/File/_Open Project...",NULL,    project_popup,    OPEN_PROJECT, NULL},
+    {"/File/_Save Project As...",NULL, project_popup,    SAVE_AS_PROJECT,NULL},
+    {"/File/_Save Project",  NULL,     project_save_cb,  0, NULL},
+    {"/File/sep1",           NULL,     NULL,             0, "<Separator>"},
 #ifdef EXPORT_PNG
     {"/File/_Export",        NULL,     NULL,             0, "<Branch>"},
     {"/File/_Export/PNG...", NULL,     export_png_popup, 0, NULL},
@@ -175,6 +194,7 @@ static GtkItemFactoryEntry popup_menu_items[] = {
     {"/Layer Color...", NULL, color_selection_popup, 0, NULL},
     {"/Load File...", NULL, load_file_popup, 0, NULL},
     {"/Unload File...", NULL, unload_file, 0, NULL},
+    {"/Invert Color", NULL, invert_color, 0, NULL},
     {"/Swap with Below", NULL, swap_layers, 1, NULL},
 };
 
@@ -204,7 +224,11 @@ create_menubar(GtkWidget *window, GtkWidget **menubar)
     gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
     
     /* Attach the new accelerator group to the window */
+#ifdef USE_GTK2
+    gtk_window_add_accel_group (GTK_WINDOW(window), accel_group);
+#else
     gtk_accel_group_attach(accel_group, GTK_OBJECT(window));
+#endif
     
     if(menubar) {
 	GtkWidget *menuEntry;
@@ -540,12 +564,26 @@ color_selection_popup(GtkWidget *widget, gpointer data)
 } /* color_selection_popup */
 
 
+static void 
+invert_color(GtkWidget *widget, gpointer data)
+{
+    if (screen.file[screen.curr_index]->inverted)
+	screen.file[screen.curr_index]->inverted = 0;
+    else
+	screen.file[screen.curr_index]->inverted = 1;
+
+    redraw_pixmap(screen.drawing_area, TRUE);
+	
+    return;
+} /* invert_color */
+
+
 static void
 cb_ok_load_file(GtkWidget *widget, GtkFileSelection *fs)
 {
     char *filename;
 
-    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
+    filename = (char *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
     if (open_image(filename, screen.curr_index, FALSE) != -1) {
 
 	/*
@@ -618,7 +656,7 @@ cb_ok_export_png(GtkWidget *widget, GtkFileSelection *fs)
     gboolean result;
     GdkWindow *window;
 
-    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
+    filename = (char *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
 
     /* This might be lengthy, show that we're busy by changing the pointer */
     window = gtk_widget_get_parent_window(widget);
@@ -695,6 +733,180 @@ export_png_popup(GtkWidget *widget, gpointer data)
 #endif /* EXPORT_PNG */
 
 
+static void
+cb_ok_project(GtkWidget *widget, gpointer data)
+{
+    char *filename = NULL;
+    project_list_t *project_list = NULL, *tmp;
+    int idx;
+    
+    if (screen.win.project) {
+	filename = (char *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(screen.win.project));
+
+	/*
+	 * Remember where we loaded file from last time
+	 */
+	if (screen.path)
+	    free(screen.path);
+	screen.path = (char *)malloc(strlen(filename) + 1);
+	strcpy(screen.path, filename);
+	dirname(screen.path);
+	screen.path = strncat(screen.path, "/", 1);
+    }
+
+    switch ((long)data) {
+    case OPEN_PROJECT:
+	
+	project_list = read_project_file(filename);
+
+	if (project_list) {
+	    load_project(project_list);
+	    /*
+	     * Save project filename for later use
+	     */
+	    if (screen.project) {
+		free(screen.project);
+		screen.project = NULL;
+	    }
+	    screen.project = (char *)malloc(strlen(filename) + 1);
+	    memset((void *)screen.project, 0, strlen(filename) + 1);
+	    strncpy(screen.project, filename, strlen(filename));
+            rename_main_window(filename, NULL);
+	    redraw_pixmap(screen.drawing_area, TRUE);
+	} else {
+	    GERB_MESSAGE("Failed to load project\n");
+	    goto cb_ok_project_end;
+	}
+	all_layers_on(NULL, NULL);
+	break;
+    case SAVE_AS_PROJECT:
+	/*
+	 * Save project filename for later use
+	 */
+	if (screen.project) {
+	    free(screen.project);
+	    screen.project = NULL;
+	}
+	screen.project = (char *)malloc(strlen(filename) + 1);
+	memset((void *)screen.project, 0, strlen(filename) + 1);
+	strncpy(screen.project, filename, strlen(filename));
+	rename_main_window(filename, NULL);
+	
+    case SAVE_PROJECT:
+	if (!screen.project) {
+	    GERB_MESSAGE("Missing project filename\n");
+	    goto cb_ok_project_end;
+	}
+	
+	if (screen.path) {
+	    project_list = (project_list_t *)malloc(sizeof(project_list_t));
+	    memset(project_list, 0, sizeof(project_list_t));
+	    project_list->next = project_list;
+	    project_list->layerno = -1;
+	    project_list->filename = screen.path;
+	    project_list->rgb[0] = screen.background->red;
+	    project_list->rgb[1] = screen.background->green;
+	    project_list->rgb[2] = screen.background->blue;
+	    project_list->next = NULL;
+	}
+	
+	for (idx = 0; idx < MAX_FILES; idx++) {
+	    if (screen.file[idx] && screen.file[idx]->name) {
+		tmp = (project_list_t *)malloc(sizeof(project_list_t));
+		memset(tmp, 0, sizeof(project_list_t));
+		tmp->next = project_list;
+		tmp->layerno = idx;
+		tmp->filename = screen.file[idx]->name;
+		tmp->rgb[0] = screen.file[idx]->color->red;
+		tmp->rgb[1] = screen.file[idx]->color->green;
+		tmp->rgb[2] = screen.file[idx]->color->blue;
+		tmp->inverted = screen.file[idx]->inverted;
+		project_list = tmp;
+	    }
+	}
+	if (write_project_file(screen.project, project_list)) {
+	    GERB_MESSAGE("Failed to write project\n");
+	    goto cb_ok_project_end;
+	}
+	break;
+    default:
+	GERB_FATAL_ERROR("Unknown operation in cb_ok_project\n");
+    }
+
+    /*
+     * Remember where we loaded file from last time
+     */
+    filename = dirname(filename);
+    if (screen.path)
+	free(screen.path);
+    screen.path = (char *)malloc(strlen(filename) + 1);
+    strcpy(screen.path, filename);
+    screen.path = strncat(screen.path, "/", 1);
+
+ cb_ok_project_end:
+    if (screen.win.project) {
+	gtk_grab_remove(screen.win.project);
+    	screen.win.project = NULL;
+    }
+
+    return;
+} /* cb_ok_project */
+
+
+static void 
+project_popup(GtkWidget *widget, gpointer data)
+{
+
+    switch ((long)data) {
+    case SAVE_AS_PROJECT:
+    case SAVE_PROJECT:
+	screen.win.project = gtk_file_selection_new("Save project filename");
+	break;
+    case OPEN_PROJECT:
+    	screen.win.project = gtk_file_selection_new("Open project filename");
+	break;
+    default:
+	GERB_FATAL_ERROR("Unknown operation in project_popup\n");
+    }
+
+    if (screen.project)
+	gtk_file_selection_set_filename
+	    (GTK_FILE_SELECTION(screen.win.project), screen.project);
+    else if (screen.path)
+	gtk_file_selection_set_filename
+	    (GTK_FILE_SELECTION(screen.win.project), screen.path);
+    
+    gtk_signal_connect
+	(GTK_OBJECT(GTK_FILE_SELECTION(screen.win.project)->ok_button),
+	 "clicked", GTK_SIGNAL_FUNC(cb_ok_project), data);
+    gtk_signal_connect_object
+	(GTK_OBJECT(GTK_FILE_SELECTION(screen.win.project)->ok_button),
+	 "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), 
+	 (gpointer)screen.win.project);
+    gtk_signal_connect_object
+	(GTK_OBJECT(GTK_FILE_SELECTION(screen.win.project)->cancel_button),
+	 "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), 
+	 (gpointer)screen.win.project);
+    
+
+    gtk_widget_show(screen.win.project);
+
+    gtk_grab_add(screen.win.project);
+    
+    return;
+} /* project_popup */
+
+static void
+project_save_cb(GtkWidget *widget, gpointer data)
+{
+
+    if (!screen.project) 
+        project_popup(widget, (gpointer) SAVE_AS_PROJECT);
+    else 
+        cb_ok_project(widget, (gpointer) SAVE_PROJECT);
+
+}
+
 static void
 all_layers_on(GtkWidget *widget, gpointer data)
 {
@@ -779,7 +991,53 @@ reload_files(GtkWidget *widget, gpointer data)
 
 
 static void
-autoscale()
+load_project(project_list_t *project_list)
+{
+    project_list_t *pl_tmp;
+    GtkStyle *defstyle, *newstyle;
+    
+    while (project_list) {
+	if (project_list->layerno == -1) {
+	    screen.background = alloc_color(project_list->rgb[0], 
+					    project_list->rgb[1],
+					    project_list->rgb[2], NULL);
+	} else {
+	    int idx =  project_list->layerno;
+
+	    if (open_image(project_list->filename, idx, FALSE) == -1)
+		exit(-1);
+
+	    /* 
+	     * Change color from default to from the project list
+	     */
+	    free(screen.file[idx]->color);
+	    screen.file[idx]->color = alloc_color(project_list->rgb[0], 
+						  project_list->rgb[1],
+						  project_list->rgb[2], NULL);
+
+	    /* 
+	     * Also change color on layer button
+	     */
+	    defstyle = gtk_widget_get_default_style();
+	    newstyle = gtk_style_copy(defstyle);
+	    newstyle->bg[GTK_STATE_NORMAL] = *(screen.file[idx]->color);
+	    newstyle->bg[GTK_STATE_ACTIVE] = *(screen.file[idx]->color);
+	    newstyle->bg[GTK_STATE_PRELIGHT] = *(screen.file[idx]->color);
+	    gtk_widget_set_style(screen.layer_button[idx], newstyle);
+
+	    screen.file[idx]->inverted = project_list->inverted;
+	}
+	pl_tmp = project_list;
+	project_list = project_list->next;
+	free(pl_tmp);
+    }
+
+    return;
+} /* load_project */
+
+
+static void
+autoscale(void)
 {
     double max_width = LONG_MIN, max_height = LONG_MIN;
     double x_scale, y_scale;
@@ -1239,6 +1497,16 @@ redraw_pixmap(GtkWidget *widget, int restart)
 	}
 	if (GTK_TOGGLE_BUTTON(screen.layer_button[i])->active &&
 	    screen.file[i]) {
+	    enum polarity_t polarity;
+
+	    if (screen.file[i]->inverted) {
+		if (screen.file[i]->image->info->polarity == POSITIVE)
+		    polarity = NEGATIVE;
+		else
+		    polarity = POSITIVE;
+	    } else {
+		polarity = screen.file[i]->image->info->polarity;
+	    }
 
 	    /*
 	     * Show progress in status bar
@@ -1263,7 +1531,7 @@ redraw_pixmap(GtkWidget *widget, int restart)
 			 screen.file[i]->image, screen.scale, 
 			 (screen.clip_bbox.x1-dmin_x)*screen.scale,
 			 (screen.clip_bbox.y1+dmax_y)*screen.scale,
-			 screen.file[i]->image->info->polarity);
+			 polarity);
 
 	    /* 
 	     * Set clipmask and draw the clipped out image onto the
@@ -1481,11 +1749,11 @@ button_press_event (GtkWidget *widget, GdkEventButton *event)
 	screen.start_y = event->y;
 	screen.centered_outline_zoom = event->state & GDK_SHIFT_MASK;
 	break;
-    case 4 :
+    case 4 : /* Scroll wheel */
 	data.z_dir = ZOOM_IN_CMOUSE;
 	do_zoom = TRUE;
 	break;
-    case 5 : 
+    case 5 :  /* Scroll wheel */
 	data.z_dir = ZOOM_OUT_CMOUSE;
 	do_zoom = TRUE;
 	break;
@@ -1502,6 +1770,39 @@ button_press_event (GtkWidget *widget, GdkEventButton *event)
 } /* button_press_event */
 
 
+#ifdef USE_GTK2
+/* Scroll wheel */
+static gint
+scroll_event(GtkWidget *widget, GdkEventScroll *event)
+{
+    gerbv_zoom_data_t data;
+
+    switch (event->direction) {
+    case GDK_SCROLL_UP:
+	data.z_dir = ZOOM_IN_CMOUSE;
+	break;
+    case GDK_SCROLL_DOWN:
+	data.z_dir = ZOOM_OUT_CMOUSE;
+	break;
+    case GDK_SCROLL_LEFT: 
+	/* Ignore */
+	return TRUE;
+    case GDK_SCROLL_RIGHT:
+	/* Ignore */
+	return TRUE;
+    default:
+	return TRUE;
+    }
+
+    /* XXX Ugly hack */
+    data.z_event = (GdkEventButton *)event;
+    zoom(widget, &data);
+
+    return TRUE;
+} /* scroll_event */
+#endif
+
+
 static gint
 button_release_event (GtkWidget *widget, GdkEventButton *event)
 {
@@ -1531,6 +1832,7 @@ static gint
 key_press_event (GtkWidget *widget, GdkEventKey *event)
 {
     GdkCursor *cursor;
+    gerbv_zoom_data_t z_data;
 
     switch (screen.state) {
     case NORMAL:
@@ -1547,6 +1849,21 @@ key_press_event (GtkWidget *widget, GdkEventKey *event)
 	    screen.state = ALT_PRESSED;
 	    screen.selected_layer = -1;
 	    break;
+	case GDK_f:
+	case GDK_F:
+	    autoscale();
+	    redraw_pixmap(screen.drawing_area, TRUE);
+	    break;
+	case GDK_z:
+	    z_data.z_dir = ZOOM_IN;
+	    zoom(widget, &z_data);
+	    break;
+	case GDK_Z:
+	    z_data.z_dir = ZOOM_OUT;
+	    zoom(widget, &z_data);
+	    break;
+	default:
+	    break;
 	}
 	break;
     case ALT_PRESSED:
@@ -1585,6 +1902,7 @@ key_press_event (GtkWidget *widget, GdkEventKey *event)
     return TRUE;
 } /* key_press_event */
 
+
 #define TOGGLE_BUTTON(button) gtk_toggle_button_set_active( \
                                       GTK_TOGGLE_BUTTON(button),\
 				 !gtk_toggle_button_get_active( \
@@ -2111,7 +2429,9 @@ static struct option longopts[] = {
     {"version",          no_argument,       NULL,    'V'},
     {"batch",            required_argument, NULL,    'b'},
     {"log",              required_argument, NULL,    'l'},
-    {"geometry",         required_argument, &longopt_val, 1 },
+    {"tools",            required_argument, NULL,    't'},
+    {"geometry",         required_argument, &longopt_val, 1},
+    {"project",          required_argument, &longopt_val, 'p'},
     /* GDK/GDK debug flags to be "let through" */
     {"gtk-module",       required_argument, &longopt_val, 2},
     {"g-fatal-warnings", no_argument,       &longopt_val, 2},
@@ -2128,6 +2448,25 @@ static struct option longopts[] = {
 };
 #endif /* HAVE_GETOPT_LONG*/
 
+void
+rename_main_window(char *filename, GtkWidget *main_win)
+{
+    char *win_title;
+    static GtkWidget *win=NULL;
+    size_t len;
+
+    if( main_win != NULL )
+	win = main_win;
+
+    assert(win != NULL);
+
+    len = strlen(WIN_TITLE) + strlen(VERSION) + 2 + strlen(filename) + 1;
+    win_title = (char *)malloc(len);
+    snprintf(win_title, len, "%s%s: %s", WIN_TITLE, VERSION, filename);
+    gtk_window_set_title(GTK_WINDOW(win), win_title);
+    free(win_title);
+				 
+}
 
 int
 main(int argc, char *argv[])
@@ -2140,9 +2479,8 @@ main(int argc, char *argv[])
     gint      screen_width, width, height;
     int       read_opt;
     int       i;
-    char      *win_title;
     int       req_width = -1, req_height = -1, req_x = 0, req_y = 0;
-    char      *rest;
+    char      *rest, *project_filename = NULL;
 
     /*
      * Setup the screen info. Must do this before getopt, since getopt
@@ -2150,14 +2488,15 @@ main(int argc, char *argv[])
      */
     memset((void *)&screen, 0, sizeof(gerbv_screen_t));
     screen.state = NORMAL;
+    screen.execpath = dirname(argv[0]);
 
     setup_init();
 	
 #ifdef HAVE_GETOPT_LONG
-    while ((read_opt = getopt_long(argc, argv, "Vl:", 
+    while ((read_opt = getopt_long(argc, argv, "Vl:p:", 
 				   longopts, &longopt_idx)) != -1) {
 #else
-    while ((read_opt = getopt(argc, argv, "Vl:")) != -1) {
+    while ((read_opt = getopt(argc, argv, "Vl:p:")) != -1) {
 #endif /* HAVE_GETOPT_LONG */
 	switch (read_opt) {
 #ifdef HAVE_GETOPT_LONG
@@ -2182,6 +2521,9 @@ main(int argc, char *argv[])
 		    break;
 		req_y = (int)strtol(rest, &rest, 10);
 		break;
+	    case 'p': /* project */
+		project_filename = optarg;
+		break;
 	    default:
 		break;
 	    }
@@ -2199,12 +2541,33 @@ main(int argc, char *argv[])
 	    setup.log.to_file = 1;
 	    setup.log.filename = optarg;
 	    break;
+	case 'p' :
+	    if (optarg == NULL) {
+		fprintf(stderr, "You must give a project filename\n");
+		exit(1);
+	    }
+	    project_filename = optarg;
+	    break;
+	case 't' :
+	    if (optarg == NULL) {
+		fprintf(stderr, "You must give a filename to read the tools from.\n");
+		exit(1);
+	    }
+	    if (!ProcessToolsFile(optarg)) {
+		fprintf(stderr, "*** ERROR processing tools file \"%s\".\n", optarg);
+		fprintf(stderr, "Make sure all lines of the file are formatted like this:\n");
+		fprintf(stderr, "T01 0.024\nT02 0.032\nT03 0.040\n...\n");
+		fprintf(stderr, "*** EXITING to prevent erroneous display.\n");
+		exit(1);
+            }
+	    break;
 	case '?':
 
 	    fprintf(stderr, "Usage : %s [FLAGS] <gerber file(s)>\n", argv[0]);
 	    fprintf(stderr, "where FLAGS could be any of\n");
 	    fprintf(stderr, "  --version|-V : Prints version of gerbv\n");
 	    fprintf(stderr, "  --log=<logfile>|-l <logfile> : Send error messages to <logfile>\n");
+	    fprintf(stderr, "  --tools=<ToolFile.drl> : Read Excellon tools from this file.\n");
 	    exit(1);
 	    break;
 	default :
@@ -2251,15 +2614,14 @@ main(int argc, char *argv[])
     /*
      * Main window 
      */
-    win_title = (char *)malloc(strlen(WIN_TITLE) + strlen(VERSION) + 1);
-    memset(win_title, 0, strlen(WIN_TITLE) + strlen(VERSION) + 1);
-    strncpy(win_title, WIN_TITLE, strlen(WIN_TITLE));
-    strncat(win_title, VERSION, strlen(VERSION));
     main_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title(GTK_WINDOW(main_win), win_title);
+    rename_main_window("", main_win);
+#ifdef USE_GTK2
+    g_signal_connect(GTK_OBJECT(main_win), "delete_event", G_CALLBACK(destroy), NULL);
+#else
     gtk_signal_connect(GTK_OBJECT(main_win), "delete_event", destroy, NULL);
     gtk_signal_connect(GTK_OBJECT(main_win), "destroy", destroy, NULL);
-    free(win_title);
+#endif
 
     /* 
      * vbox contains menubar and hbox
@@ -2297,7 +2659,11 @@ main(int argc, char *argv[])
     screen.statusbar.msg = gtk_label_new("");
     gtk_label_set_justify(GTK_LABEL(screen.statusbar.msg), GTK_JUSTIFY_LEFT);
     textStyle = gtk_style_new();
+#ifndef USE_GTK2
     textStyle->font = gdk_font_load(setup.status_fontname);
+#else
+    textStyle->font_desc = pango_font_description_from_string(setup.status_fontname);
+#endif
     gtk_widget_set_style(GTK_WIDGET(screen.statusbar.msg), textStyle);
     screen.statusbar.msgstr[0] = '\0';
     screen.statusbar.coordstr[0] = '\0';
@@ -2306,11 +2672,45 @@ main(int argc, char *argv[])
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
 
     /*
-     * Fill with files (eventually) given on command line
+     * If project is given, load that one and use it for files and colors.
+     * Else load files (eventually) given on the command line.
+     * This limits you to either give files on the commandline or just load
+     * a project.
      */
-    for(i = optind ; i < argc; i++)
-	if (open_image(argv[i], i - optind, FALSE) == -1)
-	    exit(-1);
+    if (project_filename) {
+	project_list_t *project_list;
+
+	project_list = read_project_file(project_filename);
+	
+	if (project_list) {
+	    load_project(project_list);
+	    if (screen.project) {
+		free(screen.project);
+		screen.project = NULL;
+	    }
+	    screen.project = (char *)malloc(strlen(project_filename) + 1);
+	    memset((void *)screen.project, 0, strlen(project_filename) + 1);
+	    strncpy(screen.project, project_filename, strlen(project_filename));
+	    /*
+	     * Remember where we loaded file from last time
+	     */
+	    if (screen.path)
+		free(screen.path);
+	    screen.path = (char *)malloc(strlen(project_filename) + 1);
+	    strcpy(screen.path, project_filename);
+	    dirname(screen.path);
+	    screen.path = strncat(screen.path, "/", 1);
+	    
+            rename_main_window(screen.project, NULL);
+	} else {
+	    GERB_MESSAGE("Failed to load project\n");
+	}
+
+    } else {
+	for(i = optind ; i < argc; i++)
+	    if (open_image(argv[i], i - optind, FALSE) == -1)
+		exit(-1);
+    }
 
     /*
      * Set gtk error log handler
@@ -2336,6 +2736,10 @@ main(int argc, char *argv[])
 		       GTK_SIGNAL_FUNC(key_press_event), NULL);
     gtk_signal_connect_after(GTK_OBJECT(main_win), "key_release_event",
 		       GTK_SIGNAL_FUNC(key_release_event), NULL);
+#ifdef USE_GTK2
+    gtk_signal_connect_after(GTK_OBJECT(main_win), "scroll_event",
+		       GTK_SIGNAL_FUNC(scroll_event), NULL);
+#endif
 
     gtk_widget_set_events(screen.drawing_area, GDK_EXPOSURE_MASK
 			  | GDK_LEAVE_NOTIFY_MASK
@@ -2344,15 +2748,22 @@ main(int argc, char *argv[])
 			  | GDK_KEY_PRESS_MASK
 			  | GDK_KEY_RELEASE_MASK
 			  | GDK_POINTER_MOTION_MASK
-			  | GDK_POINTER_MOTION_HINT_MASK);
-    
+			  | GDK_POINTER_MOTION_HINT_MASK
+#ifdef USE_GTK2
+			  | GDK_SCROLL_MASK
+#endif
+			  );
+
     gtk_widget_show_all(main_win);
     set_window_icon(main_win);
 
     /* It seems this has to be done after the button is shown for
        the first time, or we get a segmentation fault */
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(screen.layer_button[0]),
-				 TRUE);
+    if (project_filename)
+	all_layers_on(NULL, NULL);
+    else
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(screen.layer_button[0]),
+				     TRUE);
 
     gtk_main();
     
